'''
directoryIndexing.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

import core.data.parsers.htmlParser as htmlParser
import core.controllers.outputManager as om
from core.controllers.basePlugin.baseGrepPlugin import baseGrepPlugin
import core.data.kb.knowledgeBase as kb
import core.data.kb.vuln as vuln
from core.data.parsers.urlParser import *
from core.data.getResponseType import *

class directoryIndexing(baseGrepPlugin):
	'''
	This plugin greps every response directory indexing problems.
	  
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseGrepPlugin.__init__(self)

	def _testResponse(self, request, response):
		
		url = uri2url(response.getURL())
		htmlString = response.getBody()
		
		if isTextOrHtml(response.getHeaders()):
			for directoryIndexingString in self._getdirectoryIndexingStrings():
				if htmlString.find( directoryIndexingString ) != -1:
					v = vuln.vuln()
					v.setURL( response.getURL() )
					v.setDesc( 'The URL : ' + response.getURL() + ' has a directory indexing problem.' )
					v.setId( response.id )
					kb.kb.append( self , 'directory' , v )
					break
	
	def setOptions( self, OptionList ):
		pass
	
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/output.xsd
		'''
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def _getdirectoryIndexingStrings(self):
		dirIndexStr = []
		### TODO: verify if I need to add more values here, IIS !!!
		dirIndexStr.append("<title>Index of /")	
		dirIndexStr.append("Last modified</a>")
		dirIndexStr.append("Parent Directory</a>")
		dirIndexStr.append("Directory Listing for")
		dirIndexStr.append("<TITLE>Folder Listing.")
		dirIndexStr.append("<TITLE>Folder Listing.")
		dirIndexStr.append("- Browsing directory ")
		return dirIndexStr
		
	def end(self):
		'''
		This method is called when the plugin wont be used anymore.
		'''
		self.printUniq( kb.kb.getData( 'directoryIndexing', 'directory' ), 'URL' )
			
	def getPluginDeps( self ):
		'''
		@return: A list with the names of the plugins that should be runned before the
		current one.
		'''
		return []
	
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This plugin greps every response directory indexing problems.
		'''
