'''
fileUpload.py

Copyright 2006 Andres Riancho

This file is part of w3af, w3af.sourceforge.net .

w3af is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation version 2 of the License.

w3af is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with w3af; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

'''

import core.controllers.outputManager as om
from core.controllers.basePlugin.baseGrepPlugin import baseGrepPlugin
import core.data.kb.knowledgeBase as kb
import core.data.kb.vuln as vuln
from core.data.parsers.urlParser import *
from core.data.getResponseType import *
import re

class fileUpload(baseGrepPlugin):
	'''
	This plugin greps every page for forms with file upload capabilities.
	  
	@author: Andres Riancho ( andres.riancho@gmail.com )
	'''

	def __init__(self):
		baseGrepPlugin.__init__(self)
		self._input = re.compile('< *input(.*?)>',re.IGNORECASE)
		self._file = re.compile('type= *"file"?',re.IGNORECASE)

	def _testResponse(self, request, response):
		
		if isTextOrHtml(response.getHeaders()):
			input_res = self._input.search( response.getBody() )
			if input_res: # input tag found
				file_res = self._file.search(input_res.group())
				if file_res:
					v = vuln.vuln()
					v.setURL( response.getURL() )
					v.setId( response.id )
					v.setDesc( "The URL : " + response.getURL() + " has form with file upload capabilities." )
					kb.kb.append( self , 'fileUpload' , v )	
	
	def setOptions( self, OptionList ):
		pass
	
	def getOptionsXML(self):
		'''
		This method returns a XML containing the Options that the plugin has.
		Using this XML the framework will build a window, a menu, or some other input method to retrieve
		the info from the user. The XML has to validate against the xml schema file located at :
		w3af/core/output.xsd
		'''
		return	'<?xml version="1.0" encoding="ISO-8859-1"?>\
		<OptionList>\
		</OptionList>\
		'

	def end(self):
		'''
		This method is called when the plugin wont be used anymore.
		'''
		self.printUniq( kb.kb.getData( 'fileUpload', 'fileUpload' ), 'URL' )

	def getPluginDeps( self ):
		'''
		@return: A list with the names of the plugins that should be runned before the
		current one.
		'''
		return []
	
	def getLongDesc( self ):
		'''
		@return: A DETAILED description of the plugin functions and features.
		'''
		return '''
		This plugin greps every page for forms with file upload capabilities.
		'''
